@@ -16,25 +16,13 @@
 #include <netlink/msg.h>
 #include <netlink/utils.h>
 #include <netlink/object.h>
+#include <netlink/cache-api.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-enum {
-	NL_ACT_UNSPEC,
-	NL_ACT_NEW,
-	NL_ACT_DEL,
-	NL_ACT_GET,
-	NL_ACT_SET,
-	NL_ACT_CHANGE,
-	__NL_ACT_MAX,
-};
-
-#define NL_ACT_MAX (__NL_ACT_MAX - 1)
-
 struct nl_cache;
-typedef void (*change_func_t)(struct nl_cache *, struct nl_object *, int, void *);
 
 /**
  * @ingroup cache
@@ -71,8 +59,6 @@ extern int			nl_cache_add(struct nl_cache *,
 					     struct nl_object *);
 extern int			nl_cache_parse_and_add(struct nl_cache *,
 						       struct nl_msg *);
-extern int			nl_cache_move(struct nl_cache *,
-					      struct nl_object *);
 extern void			nl_cache_remove(struct nl_object *);
 extern int			nl_cache_refill(struct nl_sock *,
 						struct nl_cache *);
@@ -94,8 +80,8 @@ extern void			nl_cache_set_flags(struct nl_cache *, unsigned int);
 extern int			nl_cache_is_empty(struct nl_cache *);
 extern struct nl_object *	nl_cache_search(struct nl_cache *,
 						struct nl_object *);
-extern struct nl_object *nl_cache_find(struct nl_cache *,
-				       struct nl_object *);
+extern struct nl_object *	nl_cache_lookup(struct nl_cache *,
+						struct nl_object *);
 extern void			nl_cache_mark_all(struct nl_cache *);
 
 /* Dumping */
@@ -160,9 +146,6 @@ extern void			nl_cache_mngr_info(struct nl_cache_mngr *,
 						   struct nl_dump_params *);
 extern void			nl_cache_mngr_free(struct nl_cache_mngr *);
 
-extern void			nl_cache_ops_get(struct nl_cache_ops *);
-extern void			nl_cache_ops_put(struct nl_cache_ops *);
-
 #ifdef __cplusplus
 }
 #endif
