@@ -6,13 +6,25 @@
  *	License as published by the Free Software Foundation version 2.1
  *	of the License.
  *
- * Copyright (c) 2003-2008 Thomas Graf <tgraf@suug.ch>
+ * Copyright (c) 2003-2012 Thomas Graf <tgraf@suug.ch>
  */
 
 /**
- * @ingroup cache
- * @defgroup object Object
+ * @ingroup core_types
+ * @defgroup object Object (Cacheable)
+ *
+ * Generic object data type, for inheritance purposes to implement cacheable
+ * data types.
+ *
+ * Related sections in the development guide:
+ *
  * @{
+ *
+ * Header
+ * ------
+ * ~~~~{.c}
+ * #include <netlink/object.h>
+ * ~~~~
  */
 
 #include <netlink-local.h>
@@ -73,11 +85,13 @@ int nl_object_alloc_name(const char *kind, struct nl_object **result)
 {
 	struct nl_cache_ops *ops;
 
-	ops = nl_cache_ops_lookup(kind);
+	ops = nl_cache_ops_lookup_safe(kind);
 	if (!ops)
 		return -NLE_OPNOTSUPP;
 
-	if (!(*result = nl_object_alloc(ops->co_obj_ops)))
+	*result = nl_object_alloc(ops->co_obj_ops);
+	nl_cache_ops_put(ops);
+	if (!*result)
 		return -NLE_NOMEM;
 
 	return 0;
@@ -126,6 +140,23 @@ struct nl_object *nl_object_clone(struct nl_object *obj)
 	return new;
 }
 
+/**
+ * Merge a cacheable object
+ * @arg dst		object to be merged into
+ * @arg src		new object to be merged into dst
+ *
+ * @return 0 or a negative error code.
+ */
+int nl_object_update(struct nl_object *dst, struct nl_object *src)
+{
+	struct nl_object_ops *ops = obj_ops(dst);
+
+	if (ops->oo_update)
+		return ops->oo_update(dst, src);
+
+	return -NLE_OPNOTSUPP;
+}
+
 /**
  * Free a cacheable object
  * @arg obj		object to free
@@ -247,6 +278,9 @@ int nl_object_is_marked(struct nl_object *obj)
  */
 void nl_object_dump(struct nl_object *obj, struct nl_dump_params *params)
 {
+	if (params->dp_buf)
+		memset(params->dp_buf, 0, params->dp_buflen);
+
 	dump_from_ops(obj, params);
 }
 
@@ -270,14 +304,22 @@ void nl_object_dump_buf(struct nl_object *obj, char *buf, size_t len)
 int nl_object_identical(struct nl_object *a, struct nl_object *b)
 {
 	struct nl_object_ops *ops = obj_ops(a);
-	int req_attrs;
+	uint32_t req_attrs;
 
 	/* Both objects must be of same type */
 	if (ops != obj_ops(b))
 		return 0;
 
-	req_attrs = ops->oo_id_attrs;
-	if (req_attrs == ~0)
+	if (ops->oo_id_attrs_get) {
+		int req_attrs_a = ops->oo_id_attrs_get(a);
+		int req_attrs_b = ops->oo_id_attrs_get(b);
+		if (req_attrs_a != req_attrs_b)
+			return 0;
+		req_attrs = req_attrs_a;
+	} else {
+		req_attrs = ops->oo_id_attrs;
+	}
+	if (req_attrs == 0xFFFFFFFF)
 		req_attrs = a->ce_mask & b->ce_mask;
 
 	/* Both objects must provide all required attributes to uniquely
@@ -373,6 +415,27 @@ char *nl_object_attr_list(struct nl_object *obj, char *buf, size_t len)
 	return nl_object_attrs2str(obj, obj->ce_mask, buf, len);
 }
 
+/**
+ * Generate object hash key
+ * @arg obj		the object
+ * @arg hashkey		destination buffer to be used for key stream
+ * @arg hashtbl_sz	hash table size
+ *
+ * @return hash key in destination buffer
+ */
+void nl_object_keygen(struct nl_object *obj, uint32_t *hashkey,
+		      uint32_t hashtbl_sz)
+{
+	struct nl_object_ops *ops = obj_ops(obj);
+
+	if (ops->oo_keygen)
+		ops->oo_keygen(obj, hashkey, hashtbl_sz);
+	else
+		*hashkey = 0;
+
+	return;
+}
+
 /** @} */
 
 /**
@@ -380,16 +443,69 @@ char *nl_object_attr_list(struct nl_object *obj, char *buf, size_t len)
  * @{
  */
 
+/**
+ * Return number of references held
+ * @arg obj		object
+ *
+ * @return The number of references held to this object
+ */
 int nl_object_get_refcnt(struct nl_object *obj)
 {
 	return obj->ce_refcnt;
 }
 
+/**
+ * Return cache the object is associated with
+ * @arg obj		object
+ *
+ * @note The returned pointer is not protected with a reference counter,
+ *       it is your responsibility.
+ *
+ * @return Pointer to cache or NULL if not associated with a cache.
+ */
 struct nl_cache *nl_object_get_cache(struct nl_object *obj)
 {
 	return obj->ce_cache;
 }
 
+/**
+ * Return the object's type
+ * @arg obj		object
+ *
+ * FIXME: link to list of object types
+ *
+ * @return Name of the object type
+ */
+const char *nl_object_get_type(const struct nl_object *obj)
+{
+	if (!obj->ce_ops)
+		BUG();
+
+	return obj->ce_ops->oo_name;
+}
+
+/**
+ * Return the netlink message type the object was derived from
+ * @arg obj		object
+ *
+ * @return Netlink message type or 0.
+ */
+int nl_object_get_msgtype(const struct nl_object *obj)
+{
+	return obj->ce_msgtype;
+}
+
+/**
+ * Return object operations structure
+ * @arg obj		object
+ *
+ * @return Pointer to the object operations structure
+ */
+struct nl_object_ops *nl_object_get_ops(const struct nl_object *obj)
+{
+	return obj->ce_ops;
+}
+
 /** @} */
 
 /** @} */
